//#include "freertos/FreeRTOS.h"
//#include "esp_wifi.h"
//#include "esp_system.h"
//#include "esp_event.h"
//#include "esp_event_loop.h"
//#include "nvs_flash.h"
//#include "driver/gpio.h"
#include "bme280.h"
#include "bme280_defs.h"
//#include "ssd1366.h"
#include "driver/i2c.h"
#include <u8g2.h>
#include "u8g2_esp32_hal.h"
#include <esp_log.h>

void i2c_setup()
{
	printf("Setting up IC driver... ");
	//i2c_driver_install(0, I2C_MODE_MASTER, 0, 0, 0);
	i2c_config_t config;
	config.mode = I2C_MODE_MASTER;
	config.sda_io_num = 18;
	config.sda_pullup_en = GPIO_PULLUP_ENABLE;
	config.scl_io_num = 19;
	config.scl_pullup_en = GPIO_PULLUP_ENABLE;
	config.master.clk_speed = 100000;
	i2c_param_config(I2C_NUM_0, &config);
	printf("Set driver parameters... ");
	esp_err_t err = i2c_driver_install(I2C_NUM_0, I2C_MODE_MASTER, 0, 0, 0);
	if (err == ESP_OK)
		printf("Driver installed!\n");
	else if (err == ESP_ERR_INVALID_ARG)
		printf("Driver install failed, invalid arguments!\n");
	else
		printf("Driver install failed!\n");
}

void i2c_detect()
{
	printf("Scanning IC bus:\n");
	uint8_t address;
	printf("     0  1  2  3  4  5  6  7  8  9  a  b  c  d  e  f\r\n");
	for (int i = 0; i < 128; i += 16) {
		printf("%02x: ", i);
		for (int j = 0; j < 16; j++) {
			fflush(stdout);
			address = i + j;
			i2c_cmd_handle_t cmd = i2c_cmd_link_create();
			i2c_master_start(cmd);
			i2c_master_write_byte(cmd, (address << 1) | I2C_MASTER_WRITE, 0x1);
			i2c_master_stop(cmd);
			esp_err_t ret = i2c_master_cmd_begin(I2C_NUM_0, cmd, 50 / portTICK_RATE_MS);
			i2c_cmd_link_delete(cmd);
			if (ret == ESP_OK) {
				printf("%02x ", address);
			} else if (ret == ESP_ERR_TIMEOUT) {
				printf("UU ");
			} else {
				printf("-- ");
			}
		}
		printf("\r\n");
	}
}

int8_t i2c_read(uint8_t dev_id, uint8_t reg_addr, uint8_t *data, uint16_t len) {
	//printf("Reading from bus: dev_id=%x, reg_addr=%x, data=%p, length=%u\n", dev_id, reg_addr, data, len);
	/*i2c_cmd_handle_t cmd = i2c_cmd_link_create();
    i2c_master_start(cmd);
    if (data_addr != -1) {
        i2c_master_write_byte(cmd, chip_addr << 1 | WRITE_BIT, ACK_CHECK_EN);
        i2c_master_write_byte(cmd, data_addr, ACK_CHECK_EN);
        i2c_master_start(cmd);
    }
    i2c_master_write_byte(cmd, chip_addr << 1 | READ_BIT, ACK_CHECK_EN);
    if (len > 1) {
        i2c_master_read(cmd, data, len - 1, ACK_VAL);
    }
    i2c_master_read_byte(cmd, data + len - 1, NACK_VAL);
    i2c_master_stop(cmd);
    esp_err_t ret = i2c_master_cmd_begin(i2c_port, cmd, 1000 / portTICK_RATE_MS);
    i2c_cmd_link_delete(cmd);*/
	i2c_cmd_handle_t cmd = i2c_cmd_link_create();
	i2c_master_start(cmd);
	i2c_master_write_byte(cmd, dev_id << 1 | I2C_MASTER_WRITE, 1);
	i2c_master_write_byte(cmd, reg_addr, 1);
	i2c_master_start(cmd);
	i2c_master_write_byte(cmd, dev_id << 1 | I2C_MASTER_READ, 1);
	if (len > 1) {
		i2c_master_read(cmd, data, len - 1, I2C_MASTER_ACK);
	}
	i2c_master_read_byte(cmd, data + len - 1, I2C_MASTER_NACK);
	i2c_master_stop(cmd);
	i2c_master_cmd_begin(I2C_NUM_0, cmd, 500 / portTICK_RATE_MS);
	i2c_cmd_link_delete(cmd);
	return 0;
}

int8_t i2c_write(uint8_t dev_id, uint8_t reg_addr, uint8_t *data, uint16_t len) {
	//printf("Writing to bus: dev_id=%x, reg_addr=%x, data=%p, length=%u\n", dev_id, reg_addr, data, len);
	i2c_cmd_handle_t cmd = i2c_cmd_link_create();
	i2c_master_start(cmd);
	i2c_master_write_byte(cmd, (dev_id << 1) | I2C_MASTER_WRITE, 1);
	i2c_master_write_byte(cmd, reg_addr, 1);
	i2c_master_write(cmd, data, len, 1);
	i2c_master_stop(cmd);
	i2c_master_cmd_begin(I2C_NUM_0, cmd, 500 / portTICK_RATE_MS);
	i2c_cmd_link_delete(cmd);
	return 0;
}

void i2c_delay(uint32_t period) {
	vTaskDelay(period / portTICK_PERIOD_MS);
}

void i2c_shutdown()
{
	printf("Shutting down IC bus... ");
	esp_err_t err = i2c_driver_delete(I2C_NUM_0);
	if (err == ESP_ERR_INVALID_ARG)
		printf("Failed, invalid arguments!\n");
	else
		printf("Success!\n");
}

void app_main(void)
{
	i2c_setup();
	struct bme280_dev dev;
	int8_t rslt = BME280_OK;

	dev.dev_id = 0x76;
	dev.intf = BME280_I2C_INTF;
	dev.read = i2c_read;
	dev.write = i2c_write;
	dev.delay_ms = i2c_delay;

	rslt = bme280_init(&dev);

	uint8_t settings_sel;
	uint32_t req_delay;
	struct bme280_data comp_data;

	/* Recommended mode of operation: Indoor navigation */
	dev.settings.osr_h = BME280_OVERSAMPLING_1X;
	dev.settings.osr_p = BME280_OVERSAMPLING_16X;
	dev.settings.osr_t = BME280_OVERSAMPLING_2X;
	dev.settings.filter = BME280_FILTER_COEFF_16;

	settings_sel = BME280_OSR_PRESS_SEL | BME280_OSR_TEMP_SEL | BME280_OSR_HUM_SEL | BME280_FILTER_SEL;

	rslt = bme280_set_sensor_settings(settings_sel, &dev);

	/*Calculate the minimum delay required between consecutive measurement based upon the sensor enabled
	 *  and the oversampling configuration. */
	req_delay = bme280_cal_meas_delay(&(dev.settings));

	printf("Temperature, Pressure, Humidity\r\n");
	/* Continuously stream sensor data */
	while (1) {
		rslt = bme280_set_sensor_mode(BME280_FORCED_MODE, &dev);
		/* Wait for the measurement to complete and print data @25Hz */
		dev.delay_ms(req_delay);
		rslt = bme280_get_sensor_data(BME280_ALL, &comp_data, &dev);
		printf("%i c, %i hPa, %i %%\r\n", comp_data.temperature/100, comp_data.pressure/100, comp_data.humidity/1024);
		vTaskDelay(50 / portTICK_PERIOD_MS);
	}

	i2c_shutdown();

	u8g2_esp32_hal_t u8g2_esp32_hal = U8G2_ESP32_HAL_DEFAULT;
	u8g2_esp32_hal.sda   = 18;
	u8g2_esp32_hal.scl  = 19;
	u8g2_esp32_hal_init(u8g2_esp32_hal);

	u8g2_t u8g2;
	u8g2_Setup_ssd1306_i2c_128x64_vcomh0_f(&u8g2, U8G2_R0, u8g2_esp32_i2c_byte_cb,
		u8g2_esp32_gpio_and_delay_cb);

	//u8x8_SetI2CAddress(&u8g2.u8x8,0x78);
	u8x8_SetI2CAddress(&u8g2.u8x8,0x3C << 1);
	u8g2_InitDisplay(&u8g2); // send init sequence to the display, display is in sleep mode after this,
	u8g2_SetPowerSave(&u8g2, 0); // wake up display

	u8g2_SetContrast(&u8g2, 50);

	u8g2_uint_t r = 10;
	u8g2_uint_t x = r+1;
	u8g2_uint_t y = r+1;
	int8_t dx = 2;
	int8_t dy = 5;

	while (1) {
		u8g2_ClearBuffer(&u8g2);
		u8g2_DrawDisc(&u8g2, x, y, r, U8G2_DRAW_ALL);
		u8g2_DrawFrame(&u8g2, 0, 0, 128, 64);
		u8g2_SendBuffer(&u8g2);
		x = x + dx;
		y = y + dy;
		if (x <= 0+r+1 || x >= 127-r-1) {
			dx = dx * -1;
		}
		if (y <= 0+r+1 || y >= 63-r-1) {
			dy = dy * -1;
		vTaskDelay(10 / portTICK_PERIOD_MS);
		}
	}

}

